@@ -16,7 +16,9 @@ email                : marco.hugentobler at sourcepole dot com
 #ifndef QGSABSTRACTGEOMETRYV2
 #define QGSABSTRACTGEOMETRYV2
 
+#include <array>
 #include <functional>
+#include <type_traits>
 #include <QString>
 
 #include "qgis_core.h"
@@ -42,6 +44,9 @@ class QDomElement;
 class QgsGeometryPartIterator;
 class QgsGeometryConstPartIterator;
 class QgsConstWkbPtr;
+class QPainterPath;
+class QgsAbstractGeometryTransformer;
+class QgsFeedback;
 
 typedef QVector< QgsPoint > QgsPointSequence;
 #ifndef SIP_RUN
@@ -114,12 +119,14 @@ class CORE_EXPORT QgsAbstractGeometry
 
       /**
        * Maximum angle between generating radii (lines from arc center
-       * to output vertices) */
+       * to output vertices)
+      */
       MaximumAngle = 0,
 
       /**
        * Maximum distance between an arbitrary point on the original
-       * curve and closest point on its approximation. */
+       * curve and closest point on its approximation.
+      */
       MaximumDifference
     };
     Q_ENUM( SegmentationToleranceType )
@@ -186,7 +193,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * \see geometryType
      * \see wktTypeStr
      */
-    inline QgsWkbTypes::Type wkbType() const { return mWkbType; }
+    inline QgsWkbTypes::Type wkbType() const SIP_HOLDGIL { return mWkbType; }
 
     /**
      * Returns the WKT type string of the geometry.
@@ -199,7 +206,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * Returns TRUE if the geometry is 3D and contains a z-value.
      * \see isMeasure
      */
-    bool is3D() const
+    bool is3D() const SIP_HOLDGIL
     {
       return QgsWkbTypes::hasZ( mWkbType );
     }
@@ -208,7 +215,7 @@ class CORE_EXPORT QgsAbstractGeometry
      * Returns TRUE if the geometry contains m values.
      * \see is3D
      */
-    bool isMeasure() const
+    bool isMeasure() const SIP_HOLDGIL
     {
       return QgsWkbTypes::hasM( mWkbType );
     }
@@ -238,15 +245,37 @@ class CORE_EXPORT QgsAbstractGeometry
 
     //export
 
+    /**
+     * WKB export flags.
+     * \since QGIS 3.14
+     */
+    enum WkbFlag
+    {
+      FlagExportTrianglesAsPolygons = 1 << 0, //!< Triangles should be exported as polygon geometries
+    };
+    Q_DECLARE_FLAGS( WkbFlags, WkbFlag )
+
+    /**
+     * Returns the length of the QByteArray returned by asWkb()
+     *
+     * The optional \a flags argument specifies flags controlling WKB export behavior
+     *
+     * \since QGIS 3.16
+     */
+    virtual int wkbSize( QgsAbstractGeometry::WkbFlags flags = QgsAbstractGeometry::WkbFlags() ) const = 0;
+
     /**
      * Returns a WKB representation of the geometry.
+     *
+     * The optional \a flags argument specifies flags controlling WKB export behavior (since QGIS 3.14).
+     *
      * \see asWkt
      * \see asGml2
      * \see asGml3
      * \see asJson()
      * \since QGIS 3.0
      */
-    virtual QByteArray asWkb() const = 0;
+    virtual QByteArray asWkb( WkbFlags flags = QgsAbstractGeometry::WkbFlags() ) const = 0;
 
     /**
      * Returns a WKT representation of the geometry.
@@ -343,6 +372,16 @@ class CORE_EXPORT QgsAbstractGeometry
      */
     virtual void draw( QPainter &p ) const = 0;
 
+    /**
+     * Returns the geometry represented as a QPainterPath.
+     *
+     * \warning not all geometry subclasses can be represented by a QPainterPath, e.g.
+     * points and multipoint geometries will return an empty path.
+     *
+     * \since QGIS 3.16
+     */
+    virtual QPainterPath asQPainterPath() const = 0;
+
     /**
      * Returns the vertex number corresponding to a vertex \a id.
      *
@@ -521,10 +560,13 @@ class CORE_EXPORT QgsAbstractGeometry
      * If the gridified geometry could not be calculated NULLPTR will be returned.
      * It may generate an invalid geometry (in some corner cases).
      * It can also be thought as rounding the edges and it may be useful for removing errors.
+     *
      * Example:
-     * \code{.cpp}
-     * geometry->snappedToGrid(1, 1);
+     *
+     * \code{.py}
+     *   geometry.snappedToGrid(1, 1)
      * \endcode
+     *
      * In this case we use a 2D grid of 1x1 to gridify.
      * In this case, it can be thought like rounding the x and y of all the points/vertices to full units (remove all decimals).
      * \param hSpacing Horizontal spacing of the grid (x axis). 0 to disable.
@@ -649,6 +691,21 @@ class CORE_EXPORT QgsAbstractGeometry
      */
     virtual bool isValid( QString &error SIP_OUT, int flags = 0 ) const = 0;
 
+    /**
+     * Transforms the vertices from the geometry in place, using the specified geometry \a transformer
+     * object.
+     *
+     * Depending on the \a transformer used, this may result in an invalid geometry.
+     *
+     * The optional \a feedback argument can be used to cancel the transformation before it completes.
+     * If this is done, the geometry will be left in a semi-transformed state.
+     *
+     * \returns TRUE if the geometry was successfully transformed.
+     *
+     * \since QGIS 3.18
+     */
+    virtual bool transform( QgsAbstractGeometryTransformer *transformer, QgsFeedback *feedback = nullptr ) = 0;
+
 #ifndef SIP_RUN
 
     /**
@@ -680,7 +737,7 @@ class CORE_EXPORT QgsAbstractGeometry
 
     /**
      * \ingroup core
-     * The part_iterator class provides STL-style iterator for geometry parts.
+     * \brief The part_iterator class provides STL-style iterator for geometry parts.
      * \since QGIS 3.6
      */
     class CORE_EXPORT part_iterator
@@ -750,7 +807,7 @@ class CORE_EXPORT QgsAbstractGeometry
 
     /**
      * \ingroup core
-     * The part_iterator class provides STL-style iterator for const references to geometry parts.
+     * \brief The part_iterator class provides STL-style iterator for const references to geometry parts.
      * \since QGIS 3.6
      */
     class CORE_EXPORT const_part_iterator
@@ -810,7 +867,7 @@ class CORE_EXPORT QgsAbstractGeometry
 
     /**
      * \ingroup core
-     * The vertex_iterator class provides STL-style iterator for vertices.
+     * \brief The vertex_iterator class provides STL-style iterator for vertices.
      * \since QGIS 3.0
      */
     class CORE_EXPORT vertex_iterator
@@ -826,10 +883,12 @@ class CORE_EXPORT QgsAbstractGeometry
         {
           const QgsAbstractGeometry *g = nullptr;  //!< Current geometry
           int index = 0;               //!< Ptr in the current geometry
+
+          bool operator==( const Level &other ) const;
         };
 
-        Level levels[3];  //!< Stack of levels - three levels should be sufficient (e.g. part index, ring index, vertex index)
-        int depth = -1;        //!< At what depth level are we right now
+        std::array<Level, 3> levels;  //!< Stack of levels - three levels should be sufficient (e.g. part index, ring index, vertex index)
+        int depth = -1;               //!< At what depth level are we right now
 
         void digDown();   //!< Prepare the stack of levels so that it points to a leaf child geometry
 
@@ -890,7 +949,8 @@ class CORE_EXPORT QgsAbstractGeometry
      * Returns Java-style iterator for traversal of parts of the geometry. This iterator
      * can safely be used to modify parts of the geometry.
      *
-     * * Example:
+     * Example
+     *
      * \code{.py}
      *   # print the WKT representation of each part in a multi-point geometry
      *   geometry = QgsMultiPoint.fromWkt( 'MultiPoint( 0 0, 1 1, 2 2)' )
@@ -927,7 +987,8 @@ class CORE_EXPORT QgsAbstractGeometry
      * \warning The iterator returns a copy of individual vertices, and accordingly geometries cannot be
      * modified using the iterator. See transformVertices() for a safe method to modify vertices "in-place".
      *
-     * * Example:
+     * Example
+     *
      * \code{.py}
      *   # print the x and y coordinate for each vertex in a LineString
      *   geometry = QgsLineString.fromWkt( 'LineString( 0 0, 1 1, 2 2)' )
@@ -1014,55 +1075,96 @@ class CORE_EXPORT QgsAbstractGeometry
  */
 struct CORE_EXPORT QgsVertexId
 {
+
+  /**
+   * Type of vertex
+   */
   enum VertexType
   {
-    SegmentVertex = 1, //start / endpoint of a segment
-    CurveVertex
+    SegmentVertex = 1, //!< The actual start or end point of a segment
+    CurveVertex, //!< An intermediate point on a segment defining the curvature of the segment
   };
 
-  explicit QgsVertexId( int _part = -1, int _ring = -1, int _vertex = -1, VertexType _type = SegmentVertex )
-    : part( _part )
-    , ring( _ring )
-    , vertex( _vertex )
-    , type( _type )
+  /**
+   * Constructor for QgsVertexId.
+   */
+  explicit QgsVertexId( int _part = -1, int _ring = -1, int _vertex = -1, VertexType _type = SegmentVertex ) SIP_HOLDGIL
+: part( _part )
+  , ring( _ring )
+  , vertex( _vertex )
+  , type( _type )
   {}
 
   /**
    * Returns TRUE if the vertex id is valid
    */
-  bool isValid() const { return part >= 0 && ring >= 0 && vertex >= 0; }
+  bool isValid() const  SIP_HOLDGIL { return part >= 0 && ring >= 0 && vertex >= 0; }
 
-  bool operator==( QgsVertexId other ) const
+  bool operator==( QgsVertexId other ) const SIP_HOLDGIL
   {
     return part == other.part && ring == other.ring && vertex == other.vertex;
   }
-  bool operator!=( QgsVertexId other ) const
+  bool operator!=( QgsVertexId other ) const SIP_HOLDGIL
   {
     return part != other.part || ring != other.ring || vertex != other.vertex;
   }
-  bool partEqual( QgsVertexId o ) const
+
+  /**
+   * Returns TRUE if this vertex ID belongs to the same part as another vertex ID.
+   */
+  bool partEqual( QgsVertexId o ) const SIP_HOLDGIL
   {
     return part >= 0 && o.part == part;
   }
-  bool ringEqual( QgsVertexId o ) const
+
+  /**
+   * Returns TRUE if this vertex ID belongs to the same ring as another vertex ID (i.e. the part
+   * and ring number are equal).
+   */
+  bool ringEqual( QgsVertexId o ) const SIP_HOLDGIL
   {
     return partEqual( o ) && ( ring >= 0 && o.ring == ring );
   }
-  bool vertexEqual( QgsVertexId o ) const
+
+  /**
+   * Returns TRUE if this vertex ID corresponds to the same vertex as another vertex ID (i.e. the part,
+   * ring number and vertex number are equal).
+   */
+  bool vertexEqual( QgsVertexId o ) const SIP_HOLDGIL
   {
     return ringEqual( o ) && ( vertex >= 0 && o.ring == ring );
   }
-  bool isValid( const QgsAbstractGeometry *geom ) const
+
+  /**
+   * Returns TRUE if this vertex ID is valid for the specified \a geom.
+   */
+  bool isValid( const QgsAbstractGeometry *geom ) const SIP_HOLDGIL
   {
     return ( part >= 0 && part < geom->partCount() ) &&
            ( ring < geom->ringCount( part ) ) &&
            ( vertex < 0 || vertex < geom->vertexCount( part, ring ) );
   }
 
-  int part;
-  int ring;
-  int vertex;
-  VertexType type;
+  //! Part number
+  int part = -1;
+
+  //! Ring number
+  int ring = -1;
+
+  //! Vertex number
+  int vertex = -1;
+
+  //! Vertex type
+  VertexType type = SegmentVertex;
+
+#ifdef SIP_RUN
+  SIP_PYOBJECT __repr__();
+  % MethodCode
+  QString str = QStringLiteral( "<QgsVertexId: %1,%2,%3%4>" ).arg( sipCpp->part ).arg( sipCpp->ring ).arg( sipCpp->vertex ).arg( sipCpp->type == QgsVertexId::CurveVertex ? QStringLiteral( " CurveVertex" ) : QString() );
+  sipRes = PyUnicode_FromString( str.toUtf8().data() );
+  % End
+#endif
+
 };
 
 #ifndef SIP_RUN
@@ -1070,7 +1172,7 @@ struct CORE_EXPORT QgsVertexId
 template <class T>
 inline T qgsgeometry_cast( const QgsAbstractGeometry *geom )
 {
-  return const_cast<T>( reinterpret_cast<T>( 0 )->cast( geom ) );
+  return const_cast<T>( std::remove_pointer<T>::type::cast( geom ) );
 }
 
 #endif
@@ -1146,7 +1248,7 @@ class CORE_EXPORT QgsGeometryPartIterator
     }
 
     //! Find out whether there are more parts
-    bool hasNext() const
+    bool hasNext() const SIP_HOLDGIL
     {
       return g && g->parts_end() != i;
     }
@@ -1196,7 +1298,7 @@ class CORE_EXPORT QgsGeometryConstPartIterator
     }
 
     //! Find out whether there are more parts
-    bool hasNext() const
+    bool hasNext() const SIP_HOLDGIL
     {
       return g && g->const_parts_end() != i;
     }
@@ -1225,4 +1327,6 @@ class CORE_EXPORT QgsGeometryConstPartIterator
 
 };
 
+Q_DECLARE_OPERATORS_FOR_FLAGS( QgsAbstractGeometry::WkbFlags )
+
 #endif //QGSABSTRACTGEOMETRYV2
