@@ -9,19 +9,19 @@
 // or else upgrading old project file will not work
 // reliably.
 // TODO QGIS 4: remove in favor of _QGIS_VERSION
-#define VERSION "3.18.0-Zürich"
-#define _QGIS_VERSION "3.18.0-Zürich"
+#define VERSION "3.18.2-Zürich"
+#define _QGIS_VERSION "3.18.2-Zürich"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 3180
+//#define VERSION_INT 3182
 // TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
-#define VERSION_INT 31800
-#define _QGIS_VERSION_INT 31800
-#define ABISYM(x) x ## 31800
+#define VERSION_INT 31802
+#define _QGIS_VERSION_INT 31802
+#define ABISYM(x) x ## 31802
 //used in main.cpp and anywhere else where the release name is needed
 #define RELEASE_NAME "Zürich"
 
@@ -31,7 +31,7 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.18.0"
+#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.18.2"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
@@ -46,7 +46,7 @@
 
 /* #undef USING_NMAKE */
 
-/* #undef USING_NINJA */
+#define USING_NINJA
 
 #define HAVE_GUI
 
@@ -60,7 +60,7 @@
 
 #define HAVE_SERVER_PYTHON_PLUGINS
 
-/* #undef HAVE_OPENCL */
+#define HAVE_OPENCL
 
 /* #undef ENABLE_MODELTEST */
 
@@ -80,13 +80,13 @@
 
 #define HAVE_EPT
 
-/* #undef HAVE_PDAL */
-#define PDAL_VERSION ""
-#define PDAL_VERSION_MAJOR_INT 
-#define PDAL_VERSION_MINOR_INT 
-#define PDAL_VERSION_MAJOR ""
-#define PDAL_VERSION_MINOR ""
-#define PDAL_VERSION_MICRO ""
+#define HAVE_PDAL
+#define PDAL_VERSION "2.2.0"
+#define PDAL_VERSION_MAJOR_INT 2
+#define PDAL_VERSION_MINOR_INT 2
+#define PDAL_VERSION_MAJOR "2"
+#define PDAL_VERSION_MINOR "2"
+#define PDAL_VERSION_MICRO "0"
 
 #endif
 
